# Generated by roxygen2: do not edit by hand

S3method(aperm,magpie)
S3method(cbind,magpie)
S3method(head,magpie)
S3method(print,magpie)
S3method(str,magpie)
S3method(tail,magpie)
S3method(tibble::as_tibble,magpie)
export("copy.attributes<-")
export("getCells<-")
export("getComment<-")
export("getCoords<-")
export("getItems<-")
export("getNames<-")
export("getRegionList<-")
export("getRegions<-")
export("getSets<-")
export("getYears<-")
export("mselect<-")
export(addDim)
export(add_columns)
export(add_dimension)
export(as.RasterBrick)
export(as.SpatRaster)
export(as.SpatRasterDataset)
export(as.SpatVector)
export(clean_magpie)
export(collapseDim)
export(collapseNames)
export(commonYears)
export(complete_magpie)
export(convergence)
export(copy.attributes)
export(copy.magpie)
export(dimCode)
export(dimExists)
export(dimOrder)
export(dimReduce)
export(dimSums)
export(extend)
export(fulldim)
export(getCPR)
export(getCells)
export(getComment)
export(getCoords)
export(getDim)
export(getItems)
export(getNames)
export(getRegionList)
export(getRegions)
export(getSets)
export(getYears)
export(guessResolution)
export(hasCoords)
export(hasSets)
export(is.magpie)
export(is.spatial)
export(is.temporal)
export(isYear)
export(lowpass)
export(magpie_expand)
export(magpiesort)
export(magpply)
export(matchDim)
export(maxample)
export(mbind)
export(mcalc)
export(mplot)
export(mselect)
export(ncells)
export(ndata)
export(ndim)
export(new.magpie)
export(nregions)
export(nyears)
export(read.magpie)
export(read.report)
export(replace_non_finite)
export(sameDims)
export(setCells)
export(setComment)
export(setItems)
export(setYears)
export(spatRasterToDataset)
export(suppressSpecificWarnings)
export(time_interpolate)
export(unitjoin)
export(unitsplit)
export(unwrap)
export(where)
export(wrap)
export(write.magpie)
export(write.report)
export(write.report2)
exportClasses(magpie)
exportMethods("[")
exportMethods("[<-")
exportMethods(Ops)
exportMethods(as.array)
exportMethods(as.data.frame)
exportMethods(as.magpie)
exportMethods(colMeans)
exportMethods(colSums)
exportMethods(ifelse)
exportMethods(is.finite)
exportMethods(is.infinite)
exportMethods(is.na)
exportMethods(is.nan)
exportMethods(log)
exportMethods(logb)
exportMethods(pmax)
exportMethods(pmin)
exportMethods(round)
exportMethods(rowMeans)
exportMethods(rowSums)
exportMethods(setNames)
exportMethods(show)
import(data.table)
import(methods)
importFrom(abind,abind)
importFrom(data.table,as.data.table)
importFrom(data.table,melt)
importFrom(data.table,tstrsplit)
importFrom(methods,Ops)
importFrom(methods,as)
importFrom(methods,callGeneric)
importFrom(methods,is)
importFrom(methods,new)
importFrom(methods,setClass)
importFrom(methods,setGeneric)
importFrom(methods,setMethod)
importFrom(methods,signature)
importFrom(rlang,.data)
importFrom(stats,as.formula)
importFrom(utils,head)
importFrom(utils,read.table)
importFrom(utils,tail)
importFrom(utils,type.convert)
importFrom(utils,write.table)
