#' magclassdata
#'
#' General magclass-dataset
#'
#' Please do not directly access that data. It should be only used by library
#' functions.
#' @name magclassdata
#' @author Jan Philipp Dietrich
NULL
