% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/as.array.R
\docType{methods}
\name{as.array-methods}
\alias{as.array-methods}
\alias{as.array,ANY-method}
\alias{as.array,magpie-method}
\title{~~ Methods for Function as.array ~~}
\usage{
\S4method{as.array}{magpie}(x)
}
\arguments{
\item{x}{object which should be converted to an array}
}
\description{
~~ Methods for function \code{as.array} ~~
}
\section{Methods}{
 \describe{

\item{list("signature(x = \"ANY\")")}{ standard as.array-method }

\item{list("signature(x = \"magpie\")")}{ Conversion takes place just by
removing MAgPIE-object specific elements } }
}

\seealso{
Other MAgPIE-Conversions: 
\code{\link{as.RasterBrick}()},
\code{\link{as.SpatRaster}()},
\code{\link{as.SpatRasterDataset}()},
\code{\link{as.SpatVector}()},
\code{\link{as.data.frame-methods}},
\code{\link{as_tibble.magpie}()},
\code{\link{unwrap}()},
\code{\link{wrap}()}
}
\concept{MAgPIE-Conversions}
\keyword{methods}
