% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rowSums-method.R
\docType{methods}
\name{rowSums-methods}
\alias{rowSums-methods}
\alias{rowSums,ANY-method}
\alias{rowSums,magpie-method}
\alias{rowMeans-methods}
\alias{rowMeans,ANY-method}
\alias{rowMeans,magpie-method}
\title{~~ Methods for Function rowSums and rowMeans ~~}
\usage{
\S4method{rowSums}{magpie}(x, na.rm = FALSE, dims = 1, ...)
}
\arguments{
\item{x}{object on which calculation should be performed}

\item{na.rm}{logical. Should missing values (including NaN) be omitted from the calculations?}

\item{dims}{integer: Which dimensions are regarded as "rows" or "columns" to sum over. For row*,
the sum or mean is over dimensions dims+1, ...; for col* it is over dimensions 1:dims.}

\item{...}{further arguments passed to other colSums/colMeans methods}
}
\description{
~~ Methods for function \code{rowSums} and \code{rowMeans}~~
}
\section{Methods}{
 \describe{

\item{list("signature(x = \"ANY\")")}{ normal rowSums and rowMeans method }

\item{list("signature(x = \"magpie\")")}{ classical method prepared to
handle MAgPIE objects } }
}

\seealso{
Other Aggregation: 
\code{\link{colSums-methods}},
\code{\link{dimSums}()}
}
\concept{Aggregation}
\keyword{keyword(s)}
\keyword{methods}
\keyword{other}
\keyword{possible}
\keyword{~~}
